/* **********************************************************************
    Copyright 2003 Rensselaer Polytechnic Institute.

    All worldwide rights reserved. A license to use, copy, modify and
    distribute this software for noncommercial research purposes only is
    hereby granted, provided that this copyright notice and accompanying
    disclaimer is not modified or removed from the software.

    DISCLAIMER: The software is distributed" AS IS" without any express or
    implied warranty, including but not limited to, any implied warranties
    of merchantability or fitness for a particular purpose or any warrant)'
    of non-infringement of any current or pending patent rights. The authors
    of the software make no representations about the suitability of this
    software for any particular purpose. The entire risk as to the quality
    and performance of the software is with the user. Should the software
    prove defective, the user assumes the cost of all necessary servicing,
    repair or correction. In particular, neither Rensselaer Polytechnic
    Institute, nor the authors of the software are liable for any indirect,
    special, consequential, or incidental damages related to the software,
    to the maximum extent the law permits.
*/

package edu.rpi.cct.uwcal.calsvci;

import java.io.Serializable;

/** These are global parameters used by the CalSvc interface.
 *
 * @author Mike Douglass       douglm@rpi.edu
 */
public class CalSvcIPars implements Serializable {
  /** The authenticated user - null for guest
   */
  private String authUser;

  private int rights;

  /** The current user - null for guest
   */
  private String user;

  /** True if this is for public admin
   */
  private boolean publicAdmin;

  /** Non-null if this is for synchronization. Identifies the client end.
   */
  private String synchId;

  /* These are only relevant fore the public admin client.
   */
  private boolean autoCreateSponsors;
  private boolean autoDeleteSponsors;
  private boolean autoCreateLocations;
  private boolean autoDeleteLocations;

  /** True if we want debugging output
   */
  private boolean debug;

  public CalSvcIPars(String authUser,
                     int rights,
                     String user,
                     boolean publicAdmin,
                     String synchId,
                     boolean autoCreateSponsors,
                     boolean autoDeleteSponsors,
                     boolean autoCreateLocations,
                     boolean autoDeleteLocations,
                     boolean debug) {
    this.authUser = authUser;
    this.rights = rights;
    this.user = user;
    this.publicAdmin = publicAdmin;
    this.synchId = synchId;
    this.autoCreateSponsors = autoCreateSponsors;
    this.autoDeleteSponsors = autoDeleteSponsors;
    this.autoCreateLocations = autoCreateLocations;
    this.autoDeleteLocations = autoDeleteLocations;
    this.debug = debug;
  }

  public String getAuthUser() {
    return authUser;
  }

  public int getRights() {
    return rights;
  }

  public String getUser() {
    return user;
  }

  /** True if this is a public admin object..
   */
  public boolean getPublicAdmin() {
    return publicAdmin;
  }

  /** Non-null if this is for synchronization. Identifies the client end.
   */
  public String getSynchId() {
    return synchId;
  }

  /** True if we should auto-create sponsors. Some sites may wish to control
   * the creation of sponsors to enforce consistency in their use. If this
   * is true we create a sponsor as we create events. If false the sponsor
   * must already exist.
   */
  public boolean getAutoCreateSponsors() {
    return autoCreateSponsors;
  }

  /** True if we should auto-delete sponsors. Some sites may wish to control
   * the deletion of sponsors to enforce consistency in their use. If this
   * is true we delete a sponsor when it becomes unused.
   */
  public boolean getAutoDeleteSponsors() {
    return autoDeleteSponsors;
  }

  /** True if we should auto-create locations. Some sites may wish to control
   * the creation of locations to enforce consistency in their use. If this
   * is true we create a location as we create events. If false the location
   * must already exist.
   */
  public boolean getAutoCreateLocations() {
    return autoCreateLocations;
  }

  /** True if we should auto-delete locations. Some sites may wish to control
   * the deletion of locations to enforce consistency in their use. If this
   * is true we delete a location when it becomes unused.
   */
  public boolean getAutoDeleteLocations() {
    return autoDeleteLocations;
  }

  public boolean getDebug() {
    return debug;
  }
}
