/* **********************************************************************
    Copyright 2003 Rensselaer Polytechnic Institute.

    All worldwide rights reserved. A license to use, copy, modify and
    distribute this software for noncommercial research purposes only is
    hereby granted, provided that this copyright notice and accompanying
    disclaimer is not modified or removed from the software.

    DISCLAIMER: The software is distributed" AS IS" without any express or
    implied warranty, including but not limited to, any implied warranties
    of merchantability or fitness for a particular purpose or any warrant)'
    of non-infringement of any current or pending patent rights. The authors
    of the software make no representations about the suitability of this
    software for any particular purpose. The entire risk as to the quality
    and performance of the software is with the user. Should the software
    prove defective, the user assumes the cost of all necessary servicing,
    repair or correction. In particular, neither Rensselaer Polytechnic
    Institute, nor the authors of the software are liable for any indirect,
    special, consequential, or incidental damages related to the software,
    to the maximum extent the law permits.
*/

package edu.rpi.cct.uwcal.common;

import edu.washington.cac.calfacade.shared.CalFacadeException;
import edu.washington.cac.calfacade.shared.LocationVO;
import edu.washington.cac.calfacade.shared.SponsorVO;

import java.io.Serializable;
import java.net.URI;

/** This interface defines methods used to return uris which will provide access
 * to entities located somewhere in the implementing calendar system.
 *
 * <p>For example, a call to getLocationURI(loc) might return something like<br/>
 *     "http://cal.myplace.edu/ucal/locations.do?id=1234"
 *
 * <p>Implementing classes will be used by services like the synch process
 * which needs to embed usable urls in the generated Icalendar objects.
 *
 * <p>In addition, there is a corresponding method which will return a
 * dummy object representing the referenced entity. This allows services,
 * when presented with such a uri, to retrieve the actual entity without a
 * web interaction.
 *
 * @author Mike Douglass   douglm@rpi.edu
 */
public interface URIgen extends Serializable {
  public URI getLocationURI(LocationVO val) throws CalFacadeException;

  /** Attempt to create a dummy object representing the given URI.
   * Throw an exception if this is not a URI representing a location.
   *
   * <p>No web or network interactions need take place. - some implementations
   * may choose to return a real object.
   *
   * @param val     URI referencing a single location
   * @return LocationVO object with id filled in
   */
  public LocationVO getLocation(URI val) throws CalFacadeException;

  public URI getSponsorURI(SponsorVO val) throws CalFacadeException;

  /** Attempt to create a dummy object representing the given URI.
   * Throw an exception if this is not a URI representing a sponsor.
   *
   * <p>No web or network interactions need take place. - some implementations
   * may choose to return a real object.
   *
   * @param val     URI referencing a single sponsor
   * @return SponsorVO object with id filled in
   */
  public SponsorVO getSponsor(URI val) throws CalFacadeException;
}

