/* **********************************************************************
    Copyright 2003 Rensselaer Polytechnic Institute.

    All worldwide rights reserved. A license to use, copy, modify and
    distribute this software for noncommercial research purposes only is
    hereby granted, provided that this copyright notice and accompanying
    disclaimer is not modified or removed from the software.

    DISCLAIMER: The software is distributed" AS IS" without any express or
    implied warranty, including but not limited to, any implied warranties
    of merchantability or fitness for a particular purpose or any warrant)'
    of non-infringement of any current or pending patent rights. The authors
    of the software make no representations about the suitability of this
    software for any particular purpose. The entire risk as to the quality
    and performance of the software is with the user. Should the software
    prove defective, the user assumes the cost of all necessary servicing,
    repair or correction. In particular, neither Rensselaer Polytechnic
    Institute, nor the authors of the software are liable for any indirect,
    special, consequential, or incidental damages related to the software,
    to the maximum extent the law permits.
*/

package edu.rpi.cct.uwcal.icalendar;

import edu.washington.cac.calfacade.shared.MyCalendarVO;

import net.fortuna.ical4j.model.Component;
import net.fortuna.ical4j.model.Property;
//import net.fortuna.ical4j.model.property.Categories;
//import net.fortuna.ical4j.model.property.Clazz;
//import net.fortuna.ical4j.model.property.Contact;
import net.fortuna.ical4j.model.property.Created;
//import net.fortuna.ical4j.model.property.Description;
//import net.fortuna.ical4j.model.property.DtEnd;
//import net.fortuna.ical4j.model.property.DtStamp;
//import net.fortuna.ical4j.model.property.DtStart;
//import net.fortuna.ical4j.model.property.ExDate;
//import net.fortuna.ical4j.model.property.ExRule;
import net.fortuna.ical4j.model.property.LastModified;
//import net.fortuna.ical4j.model.property.Location;
//import net.fortuna.ical4j.model.property.Organizer;
//import net.fortuna.ical4j.model.property.ProdId;
//import net.fortuna.ical4j.model.property.RDate;
//import net.fortuna.ical4j.model.property.RRule;
//import net.fortuna.ical4j.model.property.Sequence;
//import net.fortuna.ical4j.model.property.Summary;
//import net.fortuna.ical4j.model.property.Uid;
//import net.fortuna.ical4j.model.property.Url;
//import net.fortuna.ical4j.model.property.Version;
import net.fortuna.ical4j.model.PropertyList;

import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;

public class ComponentWrapper {
  private Component comp;

  private PropertyList pl;

  public ComponentWrapper(Component comp) {
    this.comp = comp;
    pl = comp.getProperties();
  }

  public Timestamp getCreated() {
    Property prop = getProp(Property.CREATED);

    if (prop == null) {
      return null;
    }

    return makeSqlTimestamp(((Created)prop).getDateTime());
  }

  public String getDescription() {
    return getPval(Property.DESCRIPTION);
  }

  public Timestamp getLastModified() {
    Property prop = getProp(Property.LAST_MODIFIED);

    if (prop == null) {
      return null;
    }

    return makeSqlTimestamp(((LastModified)prop).getDateTime());
  }

  public boolean getPublic() {
    return "PUBLIC".equals(getPval(Property.CLASS));
  }

  public String getSummary() {
    return getPval(Property.SUMMARY);
  }

  /* ====================================================================
   *                   Private methods
   * ==================================================================== */

  private Property getProp(String name) {
    if (pl == null) {
      // Empty Component
      return null;
    }

    return pl.getProperty(name);
  }

  private String getPval(String name) {
    Property prop = getProp(name);

    if (prop == null) {
      return null;
    }

    return prop.getValue();
  }

  /** Make a date object that includes the date and time
   */
  private java.util.Date makeDateTime(Date dt, Time tm) {
    return new MyCalendarVO(dt, tm).getTime();
  }

  /** Get a sql timestamp object from a date-time */
  private Timestamp makeSqlTimestamp(java.util.Date dtTm) {
    return new Timestamp(dtTm.getTime());
  }

  /** Get a sql date object from a date-time */
  private Date makeSqlDate(java.util.Date dtTm) {
    return new Date(dtTm.getTime());
  }

  /** Get a sql time object from a date-time */
  private Time makeSqlTime(java.util.Date dtTm) {
    return new Time(dtTm.getTime());
  }

}

