/* **********************************************************************
    Copyright 2003 Rensselaer Polytechnic Institute.

    All worldwide rights reserved. A license to use, copy, modify and
    distribute this software for noncommercial research purposes only is
    hereby granted, provided that this copyright notice and accompanying
    disclaimer is not modified or removed from the software.

    DISCLAIMER: The software is distributed" AS IS" without any express or
    implied warranty, including but not limited to, any implied warranties
    of merchantability or fitness for a particular purpose or any warrant)'
    of non-infringement of any current or pending patent rights. The authors
    of the software make no representations about the suitability of this
    software for any particular purpose. The entire risk as to the quality
    and performance of the software is with the user. Should the software
    prove defective, the user assumes the cost of all necessary servicing,
    repair or correction. In particular, neither Rensselaer Polytechnic
    Institute, nor the authors of the software are liable for any indirect,
    special, consequential, or incidental damages related to the software,
    to the maximum extent the law permits.
*/

package edu.rpi.cct.uwcal.resources;

import java.io.Serializable;
import java.util.ResourceBundle;
import java.util.Locale;

import org.apache.log4j.Logger;

/** Object to provide internationalized resources for the calendar suite.
 *
 * @author Mike Douglass   douglm@rpi.edu
 */
public class Resources implements Serializable {
  public static final String SUBADDRESS = "subaddress";

  public static final String EMAIL = "email";

  public static final String PHONENBR = "phoneNbr";

  public static final String URL = "url";

  private static final String bundleBase = "edu.rpi.cct.uwcal.resources.UWCalResources";

  /** Bundle for the default locale */
  private ResourceBundle bundle = ResourceBundle.getBundle(bundleBase);

  private Locale defaultLocale = Locale.getDefault();

  public Resources() {}

  public Resources(Locale loc) {
    defaultLocale = loc;
    bundle = ResourceBundle.getBundle(bundleBase, loc);
  }

  public void setDefaultLocale(Locale loc) {
    defaultLocale = loc;
  }

  public String getString(String key) {
    try {
      return bundle.getString(key);
    } catch (Throwable t) {
      Logger.getLogger(Resources.class).warn("Exception getting resource " + key, t);
      return null;
    }
  }

  public static String getString(String key, Locale loc) {
    try {
      return ResourceBundle.getBundle(bundleBase, loc).getString(key);
    } catch (Throwable t) {
      Logger.getLogger(Resources.class).warn("Exception getting resource " + key, t);
      return null;
    }
  }
}
