/* **********************************************************************
    Copyright 2003 Rensselaer Polytechnic Institute.

    All worldwide rights reserved. A license to use, copy, modify and
    distribute this software for noncommercial research purposes only is
    hereby granted, provided that this copyright notice and accompanying
    disclaimer is not modified or removed from the software.

    DISCLAIMER: The software is distributed" AS IS" without any express or
    implied warranty, including but not limited to, any implied warranties
    of merchantability or fitness for a particular purpose or any warrant)'
    of non-infringement of any current or pending patent rights. The authors
    of the software make no representations about the suitability of this
    software for any particular purpose. The entire risk as to the quality
    and performance of the software is with the user. Should the software
    prove defective, the user assumes the cost of all necessary servicing,
    repair or correction. In particular, neither Rensselaer Polytechnic
    Institute, nor the authors of the software are liable for any indirect,
    special, consequential, or incidental damages related to the software,
    to the maximum extent the law permits.
*/

package edu.rpi.cct.uwcal.webcommon;

import edu.rpi.cct.uwcal.calsvci.CalSvcI;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletRequestEvent;
import javax.servlet.ServletRequestListener;
import org.apache.log4j.Logger;

/** This class must be installed as a request listener for a UWCal web application.
 *
 * <p>We assume that any CalSvci object must remain open until after the jsp
 * has done its stuff, i.e. after the action returns but before we finally
 * deliver the response. This listener uses a callback object stored as an
 * attribute in the session.
 *
 * @author Mike Douglass douglm@rpi.edu
 */
public class UWCalRequestListener implements ServletRequestListener {
  public void requestDestroyed(ServletRequestEvent sre) {
getLogger().info("requestDestroyed entry");
    HttpServletRequest req = (HttpServletRequest)sre.getServletRequest();
    HttpSession sess = req.getSession();

    UWCalCallback cb = (UWCalCallback)sess.getAttribute(UWCalCallback.cbAttrName);

    try {
      if (cb != null) {
        cb.out();
      }
    } catch (Throwable t) {
      getLogger().error("Callback exception: ", t);
      throw new RuntimeException(t);
    } finally {
      cb = (UWCalCallback)sess.getAttribute(UWCalCallback.cbAttrName);

      if (cb != null) {
        try {
          cb.close();
        } catch (Throwable t) {
          getLogger().error("Callback exception: ", t);
          throw new RuntimeException(t);
        }
      }
    }
  }

  public void requestInitialized(ServletRequestEvent sre) {
getLogger().info("requestInitialized entry");
    HttpServletRequest req = (HttpServletRequest)sre.getServletRequest();
    HttpSession sess = req.getSession();

    UWCalCallback cb = (UWCalCallback)sess.getAttribute(UWCalCallback.cbAttrName);

    try {
      if (cb != null) {
        cb.in();
      }
    } catch (Throwable t) {
      getLogger().error("Callback exception: ", t);
      throw new RuntimeException(t);
    }
  }

  private Logger getLogger() {
    return Logger.getLogger(this.getClass());
  }
}


