/* **********************************************************************
    Copyright 2003 Rensselaer Polytechnic Institute.

    All worldwide rights reserved. A license to use, copy, modify and
    distribute this software for noncommercial research purposes only is
    hereby granted, provided that this copyright notice and accompanying
    disclaimer is not modified or removed from the software.

    DISCLAIMER: The software is distributed" AS IS" without any express or
    implied warranty, including but not limited to, any implied warranties
    of merchantability or fitness for a particular purpose or any warrant)'
    of non-infringement of any current or pending patent rights. The authors
    of the software make no representations about the suitability of this
    software for any particular purpose. The entire risk as to the quality
    and performance of the software is with the user. Should the software
    prove defective, the user assumes the cost of all necessary servicing,
    repair or correction. In particular, neither Rensselaer Polytechnic
    Institute, nor the authors of the software are liable for any indirect,
    special, consequential, or incidental damages related to the software,
    to the maximum extent the law permits.
*/

package edu.rpi.cct.uwcal.webcommon;

import edu.rpi.cct.uwcal.calsvci.CalSvcI;

import java.io.IOException;
import java.lang.IllegalStateException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import org.apache.log4j.Logger;

/** This class must be installed as a filter for a UWCal web application.
 *
 * <p>We assume that any CalSvci object must remain open until after the jsp
 * has done its stuff, i.e. after the action returns but before we finally
 * deliver the response. This filter uses a callback object stored as an
 * attribute in the session.
 *
 * @author Mike Douglass douglm@rpi.edu
 */
public class UWCalSvciFilter implements Filter {
  public void init(FilterConfig cnfg) throws ServletException {
  }

  public void doFilter(ServletRequest req,
                       ServletResponse resp,
                       FilterChain chain)
          throws IOException, ServletException {
    HttpSession sess = ((HttpServletRequest)req).getSession();
    UWCalCallback cb = null;

    try {
      cb = getCb(sess);

      if (cb != null) {
        cb.in();
      }

      chain.doFilter(req, resp);

      cb = getCb(sess);
      if (cb != null) {
        cb.out();
      }
    } catch (Throwable t) {
      Logger.getLogger(this.getClass()).error("Callback exception: ", t);
      throw new ServletException(t);
    } finally {
      try {
        cb = getCb(sess);

        if (cb != null) {
          cb.close();
        }
      } catch (Throwable t) {
        Logger.getLogger(this.getClass()).error("Callback exception: ", t);
        throw new ServletException(t);
      }
    }
  }

  public void destroy() {
  }

  private UWCalCallback getCb(HttpSession sess) throws Throwable {
    if (sess == null) {
      return null;
    }

    try {
      return (UWCalCallback)sess.getAttribute(UWCalCallback.cbAttrName);
    } catch (IllegalStateException ise) {
      // Invalidated session - assume logged out
      return null;
    }
  }
}


