/* **********************************************************************
    Copyright 2003 Rensselaer Polytechnic Institute.

    All worldwide rights reserved. A license to use, copy, modify and
    distribute this software for noncommercial research purposes only is
    hereby granted, provided that this copyright notice and accompanying
    disclaimer is not modified or removed from the software.

    DISCLAIMER: The software is distributed" AS IS" without any express or
    implied warranty, including but not limited to, any implied warranties
    of merchantability or fitness for a particular purpose or any warrant)'
    of non-infringement of any current or pending patent rights. The authors
    of the software make no representations about the suitability of this
    software for any particular purpose. The entire risk as to the quality
    and performance of the software is with the user. Should the software
    prove defective, the user assumes the cost of all necessary servicing,
    repair or correction. In particular, neither Rensselaer Polytechnic
    Institute, nor the authors of the software are liable for any indirect,
    special, consequential, or incidental damages related to the software,
    to the maximum extent the law permits.
*/

package edu.rpi.sss.util.servlets.io;

import java.io.PrintWriter;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;
import javax.servlet.ServletContext;
import javax.servlet.ServletOutputStream;

/** This class provides a useful form of the wrapped response.
 */
public class WrappedResponse extends HttpServletResponseWrapper {
  protected ServletContext ctx;

  protected boolean debug = false;

  protected boolean transformNeeded = false;

  final ByteArrayPrintWriter pw = new ByteArrayPrintWriter();

  public WrappedResponse(HttpServletResponse response,
                         ServletContext ctx, boolean debug) {
    super(response);
    this.ctx = ctx;
    this.debug = debug;
  }

  public PrintWriter getWriter() {
    return pw.getWriter();
  }

  public ServletOutputStream getOutputStream() {
    return pw.getStream();
  }

  public void flushBuffer() {
  }

  public void setContentType(String type) {
    getResponse().setContentType(type);
  }

  public void setTransformNeeded(boolean val) {
    transformNeeded = val;
  }

  public boolean getTransformNeeded() {
    return transformNeeded;
  }

  public byte[] toByteArray() {
    if (pw == null) {
      return null;
    }

    return pw.toByteArray();
  }

  public void close() {
  }
}

