/* **********************************************************************
    Copyright 2003 Rensselaer Polytechnic Institute.

    All worldwide rights reserved. A license to use, copy, modify and
    distribute this software for noncommercial research purposes only is
    hereby granted, provided that this copyright notice and accompanying
    disclaimer is not modified or removed from the software.

    DISCLAIMER: The software is distributed" AS IS" without any express or
    implied warranty, including but not limited to, any implied warranties
    of merchantability or fitness for a particular purpose or any warrant)'
    of non-infringement of any current or pending patent rights. The authors
    of the software make no representations about the suitability of this
    software for any particular purpose. The entire risk as to the quality
    and performance of the software is with the user. Should the software
    prove defective, the user assumes the cost of all necessary servicing,
    repair or correction. In particular, neither Rensselaer Polytechnic
    Institute, nor the authors of the software are liable for any indirect,
    special, consequential, or incidental damages related to the software,
    to the maximum extent the law permits.
*/

package edu.washington.cac.calfacade.shared;

import java.util.ArrayList;

/** Class which represents an integer array which can be stored as a string
*
* @author   Mike Douglass
* @version  1.0
*
* A number of bitty utility routines.
*/
public class IntArrayList extends ArrayList {
  public IntArrayList() {
    super();
  }

  public IntArrayList(int initialCapacity) {
    super(initialCapacity);
  }

  public int getInt(int i) {
    Object o = get(i);

    if (!(o instanceof Integer)) {
      throw new RuntimeException("Invalid IntArray element class");
    }

    return ((Integer)o).intValue();
  }

  public void add(int val) {
    add(new Integer(val));
  }

  /** Return a String representing the array, achieved by concatenating elements
   * with intervening blanks.
   *
   * @return String encoded value
   */
  public String encode(){
    int sz = size();

    if (sz == 0) {
      return "";
    }

    StringBuffer sb = new StringBuffer();

    for (int i = 0; i < sz; i++) {
      if (i > 0) {
        sb.append(" ");
      }

      sb.append(getInt(i));
    }

    return sb.toString();
  }

  /** Return an IntArray resulting from decoding the given String which
   * should have been encoded by encode
   *
   * @param  val      String value encoded by encode
   * @return IntArrayList decoded value
   */
  public static IntArrayList decode(String val){
    IntArrayList ia = new IntArrayList();

    if (val == null) {
      return ia;
    }

    int len = val.length();

    if (len == 0) {
      return ia;
    }

    int i = 0;

    while (i < len) {
      int end = val.indexOf(" ", i);

      String s;
      if (end < 0) {
        s = val.substring(i);
        i = len;
      } else {
        s = val.substring(i, end);
        i = end + 1;
      }

      try {
        ia.add(Integer.valueOf(s));
      } catch (Throwable t) {
        throw new RuntimeException(t);
      }
    }

    return ia;
  }
}
