package edu.washington.cac.calfacade.shared;

import java.io.Serializable;
import java.util.Comparator;

/** Base object which is closest to the db representation.
 * Possibly used for ejb or hibernate mapping generation.
 *
 * <p>Keyword attribute
 *
 *  @version 1.0
 */
public class KeywordAttrVO implements Comparable, Comparator, Serializable {
  public static final String noValueCheck = "*NOVAL*";

  protected int id = CalFacadeDefs.unsavedItemKey;

  /** The owning keyword */
  protected KeywordVO keyword;

  protected String name;
  protected String val;

  public KeywordAttrVO() {
  }

  public KeywordAttrVO(String name, String val) {
    this.name = name;
    this.val = val;
  }

  public KeywordAttrVO(int id, String name, String val) {
    setId(id);
    setName(name);
    setVal(val);
  }

  /** ===================================================================
   *                   Bean methods
   *  =================================================================== */

  /**
    Set the id for this keyword
    @param val    int Keyword id
   */
  public void setId(int val) {
    id = val;
  }

  /** Get the kwywords unique id
   *
   * @return int    the keywords unique id
   */
  public int getId() {
    return id;
  }

  /** Set the corresponding keyword for this keyword attributes entry
   *
   * @param val    KeywordVO owning this object
   */
  public void setKeyword(KeywordVO val) {
    keyword = val;
  }

  /** Get the owning KeywordVO
   *
   * @return KeywordVO    the keyword
   */
  public KeywordVO getKeyword() {
    return keyword;
  }

  /** Set the name
   *
   * @param val    String name
   */
  public void setName(String val) {
    name = val;
  }

  /** Get the name
   *
   * @return String   name
   */
  public String getName() {
    return name;
  }

  public void setVal(String val) {
    this.val = val;
  }

  public String getVal() {
    return val;
  }

  /** ===================================================================
   *                   Convenience methods
   *  =================================================================== */

  /** We consider the attribute a match if the name and value match.
   * If one of the values is the string "*NOVAL*" we don't check values.
   * This allows us a simple check on categories for example.
   */
  public boolean matches(String aname, String aval) {
    if (name == null) {
      if (aname != null) {
        return false;
      }
    } else if (!name.equals(aname)) {
      return false;
    }

    if (val == null) {
      if (aval != null) {
        return aval.equals(noValueCheck);
      }
    } else if (val.equals(noValueCheck)) {
      return true;
    } else if (aval.equals(noValueCheck)) {
      return true;
    } else if (!val.equals(aval)) {
      return false;
    }

    return true;
  }

  /** ===================================================================
   *                   Object methods
   *  =================================================================== */

  /** We'll provide a compare method based on the name and value.
   */
  public int compare(Object o1, Object o2) {
    if (!(o1 instanceof KeywordAttrVO)) {
      return -1;
    }

    if (!(o2 instanceof KeywordAttrVO)) {
      return 1;
    }

    KeywordAttrVO ka1 = (KeywordAttrVO)o1;
    KeywordAttrVO ka2 = (KeywordAttrVO)o2;

    int cmp = CalFacadeUtil.compareStrings(ka1.name, ka2.name);
    if (cmp != 0) {
      return cmp;
    }

    return CalFacadeUtil.compareStrings(ka1.val, ka2.val);
  }

  public int hashCode() {
    int hc = 1;

    if (name != null) {
      hc *= name.hashCode();
    }

    if (val != null) {
      hc *= val.hashCode();
    }

    return hc;
  }

  /** Use our compare method.
   */
  public int compareTo(Object o2) {
    return compare(this, o2);
  }

  /** see matches for how we compare objects
   *
   */
  public boolean equals(Object obj) {
    if (!(obj instanceof KeywordAttrVO)) {
      return false;
    }

    KeywordAttrVO that = (KeywordAttrVO)obj;

    return matches(that.name, that.val);
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append("KeywordAttrVO{id=");
    sb.append(id);
    sb.append(", name=");
    sb.append(name);
    sb.append(", val=");
    sb.append(val);
    sb.append("}");

    return sb.toString();
  }
}
