package edu.washington.cac.calfacade.shared.filter;

import edu.washington.cac.calfacade.shared.EventVO;
import edu.washington.cac.calfacade.shared.UserVO;

/** A filter that selects all events created by a particular user
 *
 * @author Greg Barnes
 * @version 1.0
 */
public class CreatorFilterVO extends FilterVO {
  /** User whose events we want to see */
  protected UserVO creator;

  /** Get a human-readable name for the calendar represented by the filter
   *
   * @return String  a human-readable name for the filter
   */
  public String getName() {
    if (name != null) {
      return name;
    }

    return "Events created by " + this.creator;
  }

  /** Set the creator to filter on
   *
   *  @param val     UserVO creator to filter on
   */
  public void setCreator(UserVO val) {
    creator = val;
  }

  /** Get the creator we're filtering on
   *  @return UserVO     creator we're filtering on
   */
  public UserVO getCreator() {
    return creator;
  }

  /** Test if this event passes the test
   *
   * @param             e Event to test
   * @return boolean    true  if the event was created by the user
   */
  public boolean match(EventVO e) {
    return e.getCreator().equals(creator);
  }

  /* ====================================================================
   *                   Copying methods
   * ==================================================================== */

  protected void copyTo(CreatorFilterVO val) {
    super.copyTo(val);
    val.setCreator((UserVO)getCreator().clone());
  }

  /* ====================================================================
   *                   Object methods
   * ==================================================================== */

  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append("CreatorFilterVO{id=");
    sb.append(id);
    sb.append(", name=");
    sb.append(getName());
    sb.append(", creator=");
    sb.append(creator.toString());
    sb.append("}");

    return sb.toString();
  }

  public Object clone() {
    CreatorFilterVO f = new CreatorFilterVO();

    copyTo(f);

    return f;
  }
}
