package edu.washington.cac.calfacade.shared.filter;

import java.util.Iterator;
import java.util.NoSuchElementException;

import edu.washington.cac.calfacade.shared.EventVO;

/**
  An iterator that filters another iterator, yielding only
  the events the filter selects

  @author Greg Barnes
  @version 1.0
 */
public class FilteredIterator implements Iterator
{
  /** The Filter to apply */
  protected FilterVO filter;

  /** The unfiltered iterator */
  private Iterator iterator;

  /** next matching element */
  private EventVO nextEvent;

  /**
    Create a new instance
    @param iterator Iterator.  Each element of the iterator should
      be an Event, or ClassCastExceptions will result
    @param filter Filter to apply to the iterator
   */
  public FilteredIterator(Iterator iterator, FilterVO filter)
  {
    this.iterator = iterator;
    this.filter = filter;
    this.nextEvent = nextMatch();  // set up for next call
  }

  /**
    @return The next event in the iterator that matches the filter,
      or null if there are no more
   */
  private EventVO nextMatch()
  {
    while (iterator.hasNext()) {
      EventVO e = (EventVO) iterator.next();

      if (filter.match(e)) {
        return e;
      }
    }

    return null;
  }

  /**
    Are there more elements?
    @return Are there more elements?
   */
  public boolean hasNext()
  {
    return nextEvent != null;
  }

  /**
    Get The next element in the iterator
    @return The next element in the iterator
    @exception NoSuchElementException If there are no more elements left
   */
  public Object next() throws NoSuchElementException
  {
    Object o = nextEvent;

    if (hasNext()) {
      nextEvent = nextMatch();  // set up for next call
      return o;
    }

    throw new NoSuchElementException();
  }

  /**
    unsupported
    @exception UnsupportedOperationException always thrown
   */
  public void remove() throws UnsupportedOperationException
  {
    throw new UnsupportedOperationException();
  }

  /** ===================================================================
   *                   Object methods
   *  =================================================================== */

  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append("FilteredIterator{filter=");
    sb.append(filter);
    sb.append("}");

    return sb.toString();
  }
}
