package edu.washington.cac.calfacade.shared.filter;

import edu.washington.cac.calfacade.shared.EventVO;
import edu.washington.cac.calfacade.shared.KeywordVO;
import edu.washington.cac.calfacade.shared.UserVO;

/**
  A filter that selects events that have a certain keyword

  @author Greg Barnes
  @version 1.0
 */
public class KeywordFilterVO extends FilterVO {
  /** The keyword to match */
  protected KeywordVO k;

  public KeywordFilterVO() {}

  public KeywordFilterVO(String name, FilterVO parent,
                         KeywordVO k) {
    super(name,  parent);
    this.k = k;
  }

  public KeywordFilterVO(int id, String name, FilterVO parent,
                         KeywordVO k) {
    super(id, name,  parent);
    this.k = k;
  }

  /** Get a human-readable name for the calendar represented by the filter
   *
   * @return String  a human-readable name for the filter
   */
  public String getName() {
    if (name != null) {
      return name;
    }

    return k.getWord();
  }

  /** Set the keyword to filter on
   *
   *  @param val     KeywordVO keyword to filter on
   */
  public void setKeyword(KeywordVO val) {
    k = val;
  }

  /** Get the keyword we're filtering on
   *  @return KeywordVO     keyword we're filtering on
   */
  public KeywordVO getKeyword() {
    return k;
  }

  /** Test if this event passes the test
   *
   * @param             e Event to test
   * @return boolean    true if the filter selects the event
   */
  public boolean match(EventVO e) {
    return e.hasKeyword(k);
  }

  /* ====================================================================
   *                   Copying methods
   * ==================================================================== */

  protected void copyTo(KeywordFilterVO val) {
    super.copyTo(val);
    val.setKeyword((KeywordVO)getKeyword().clone());
  }

  /* ====================================================================
   *                   Object methods
   * ==================================================================== */

  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append("KeywordFilterVO{id=");
    sb.append(id);
    sb.append(", name=");
    sb.append(getName());
    sb.append(", keyword=");
    sb.append(k);
    sb.append("}");

    return sb.toString();
  }

  public Object clone() {
    KeywordFilterVO f = new KeywordFilterVO();

    copyTo(f);

    return f;
  }
}
