package edu.washington.cac.calfacade.shared.filter;

import edu.washington.cac.calfacade.shared.EventVO;
import edu.washington.cac.calfacade.shared.LocationVO;

/**
  A filter that selects events that have a particular location

  @author Greg Barnes
  @version 1.0
 */
public class LocationFilterVO extends FilterVO {
  /** The Location to match */
  protected LocationVO l;

  /** Get a human-readable name for the calendar represented by the filter
   *
   * @return String  a human-readable name for the filter
   */
  public String getName() {
    if (name != null) {
      return name;
    }

    return l.getAddress();
  }

  /** Set the location to filter on
   *
   *  @param val     LocationVO location to filter on
   */
  public void setLocation(LocationVO val) {
    l = val;
  }

  /** Get the location we're filtering on
   *  @return LocationVO     location we're filtering on
   */
  public LocationVO getLocation() {
    return l;
  }

  /** Test if this event passes the test
   *
   * @param             e Event to test
   * @return boolean    true  if the event passes
   */
  public boolean match(EventVO e) {
    /* If a public location is entered in a slightly different way (e.g.,
       with a different address), a new location is created.  Matching
       on ID, therefore, will probably miss many valid matches, so we
       match on name
     */
    return l.getAddress().equals(
           e.getLocation().getAddress());
  }

  /* ====================================================================
   *                   Copying methods
   * ==================================================================== */

  protected void copyTo(LocationFilterVO val) {
    super.copyTo(val);
    val.setLocation((LocationVO)getLocation().clone());
  }

  /* ====================================================================
   *                   Object methods
   * ==================================================================== */

  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append("LocationFilterVO{id=");
    sb.append(id);
    sb.append(", name=");
    sb.append(getName());
    sb.append(", location=");
    sb.append(l);
    sb.append("}");

    return sb.toString();
  }

  public Object clone() {
    LocationFilterVO f = new LocationFilterVO();

    copyTo(f);

    return f;
  }
}
