package edu.washington.cac.calfacade.shared.filter;

import java.util.Iterator;

import edu.washington.cac.calfacade.shared.EventVO;
import edu.washington.cac.calfacade.shared.KeywordVO;

/**
  A filter that selects events that contain a string in certain fields

  @author Greg Barnes
  @version 1.0
 */
public class SearchFilterVO extends FilterVO
{
  /** The String to match */
  protected String s;

  /**
    Create a new instance
   */
  public SearchFilterVO() {
  }

  /**
    Create a new instance
    @param s String to match
   */
  public SearchFilterVO(String s) {
    this.s = s;
  }

  /**
    Get an internal name for the calendar represented by the filter
    @return an internal name for the calendar represented by the filter
   */
  public String getName() {
    if (name != null) {
      return name;
    }

    return "search:" + s;
  }

  /**
    Get a human-readable name for the calendar represented by the filter
    @return a human-readable name for the calendar represented by the filter
   */
  public String getTitle() {
    if (title != null) {
      return title;
    }

    return s;
  }

  public void setSearchTerm(String val) {
    s = val;
  }

  public String getSearchTerm() {
    return s;
  }

  /** Test if this event passes the test
   *
   * @param             e Event to test
   * @return boolean    true  if the event was created by the user
   */
  public boolean match(EventVO e) {
    if (contains(e.getShortdesc(), s) ||
        contains(e.getLongdesc(), s) ||
        contains(e.getSponsor().getName(), s) ||
        contains(e.getLocation().getAddress(), s) ||
        contains(e.getLocation().getSubaddress(), s) ||
        contains(e.getSponsor().getEmail(), s) ||
        contains(e.getSponsor().getPhone(), s) ||
        contains(e.getCost(), s) )
    {
      return true;
    }

    Iterator it = e.getKeywords().iterator();

    while (it.hasNext()) {
      KeywordVO k = (KeywordVO)it.next();

      if (contains(k.getWord(), s) || contains(k.getDescription(), s)) {
        return true;
      }
    }

    return false;
  }

  /**
    Does string #1 contain string #2 as a substring?
    @param s1 String #1
    @param s2 String #2
    @return Does string #1 contain string #2 as a substring?
   */
  private boolean contains(String s1, String s2)
  {
    return s1 != null && s1.toLowerCase().indexOf(s2.toLowerCase()) != -1;
  }

  /* ====================================================================
   *                   Copying methods
   * ==================================================================== */

  protected void copyTo(SearchFilterVO val) {
    super.copyTo(val);
    val.setSearchTerm(getSearchTerm());
  }

  /* ====================================================================
   *                   Object methods
   * ==================================================================== */

  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append("SearchFilterVO{id=");
    sb.append(id);
    sb.append(", name=");
    sb.append(getName());
    sb.append(", term=");
    sb.append(s);
    sb.append("}");

    return sb.toString();
  }

  public Object clone() {
    SearchFilterVO f = new SearchFilterVO();

    copyTo(f);

    return f;
  }
}
