package edu.washington.cac.calfacade.shared.filter;

import edu.washington.cac.calfacade.shared.EventVO;
import edu.washington.cac.calfacade.shared.SponsorVO;

/**
  A filter that selects events that have a certain sponsor

  @author Greg Barnes
  @version 1.0
 */
public class SponsorFilterVO extends FilterVO {
  /** The Sponsor to match */
  protected SponsorVO s;

  /**
    Get an internal name for the calendar represented by the filter
    @return an internal name for the calendar represented by the filter
   */
  public String getName() {
    if (name != null) {
      return name;
    }

    //%%% this could fail miserably if the filter was constructed by passing
    //%%% in a bogus sponsor with the correct name
    return "Sponsor:" + s.getId();
  }

  /**
    Get a human-readable name for the calendar represented by the filter
    @return a human-readable name for the calendar represented by the filter
   */
  public String getTitle() {
    if (name != null) {
      return title;
    }
    return s.getName();
  }

  /** Set the sponsor to filter on
   *
   *  @param val     SponsorVO sponsor to filter on
   */
  public void setSponsor(SponsorVO val) {
    s = val;
  }

  /** Get the sponsor we're filtering on
   *  @return SponsorVO     sponsor we're filtering on
   */
  public SponsorVO getSponsor() {
    return s;
  }

  /**
    @return If the filter selects the event
    @param e Event to test
   */
  public boolean match(EventVO e) {
    /* If a public sponsor is entered in a slightly different way (e.g.,
       with a different phone number), a new sponsor is created.  Matching
       on ID, therefore, will probably miss many valid matches, so we
       match on name
     */
    return s.getName().equals(e.getSponsor().getName());
  }

  /* ====================================================================
   *                   Copying methods
   * ==================================================================== */

  protected void copyTo(SponsorFilterVO val) {
    super.copyTo(val);
    val.setSponsor((SponsorVO)getSponsor().clone());
  }

  /* ====================================================================
   *                   Object methods
   * ==================================================================== */

  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append("SponsorFilterVO{id=");
    sb.append(id);
    sb.append(", name=");
    sb.append(getName());
    sb.append(", sponsor=");
    sb.append(s);
    sb.append("}");

    return sb.toString();
  }

  public Object clone() {
    SponsorFilterVO f = new SponsorFilterVO();

    copyTo(f);

    return f;
  }
}
