/* **********************************************************************
    Copyright 2003 Rensselaer Polytechnic Institute.

    All worldwide rights reserved. A license to use, copy, modify and
    distribute this software for noncommercial research purposes only is
    hereby granted, provided that this copyright notice and accompanying
    disclaimer is not modified or removed from the software.

    DISCLAIMER: The software is distributed" AS IS" without any express or
    implied warranty, including but not limited to, any implied warranties
    of merchantability or fitness for a particular purpose or any warrant)'
    of non-infringement of any current or pending patent rights. The authors
    of the software make no representations about the suitability of this
    software for any particular purpose. The entire risk as to the quality
    and performance of the software is with the user. Should the software
    prove defective, the user assumes the cost of all necessary servicing,
    repair or correction. In particular, neither Rensselaer Polytechnic
    Institute, nor the authors of the software are liable for any indirect,
    special, consequential, or incidental damages related to the software,
    to the maximum extent the law permits.
*/

package edu.tests.rpi.cct.uwcal.calsvci;

import edu.rpi.cct.uwcal.calsvci.CalSvcI;

import edu.washington.cac.calfacade.shared.CalFacadeDefs;
import edu.washington.cac.calfacade.shared.UserAuth;
import edu.washington.cac.calfacade.shared.UserVO;

import junit.framework.TestCase;

/** Test the CalSvcI class
 *
 * <p>We require that the UserAuth classes function enough for
 * this test framework. I'm not sure we can test the roles based version for
 * example, as this requires a servlet.
 *
 * <p>The assumption we're making here is that we don't need to pass any
 * object to the class. It's probably a db based version.
 *
 * @author Mike Douglass       douglm@rpi.edu
 */
public class CalSvcTest extends TestCase {
  private CalSvcI svci;

  private CalSvcTestUtil svciUtil;

  private boolean debug = true;

  private static String user = "caladmin";

  public void testCalSvc() {
    try {
      svciUtil = new CalSvcTestUtil(debug);

      svciUtil.getSvci(user, UserAuth.superUser, true);

      /* Add a bunch of public locations */
      int numLocs = 10;
      int firstLoc = addLocations(numLocs, true);

      /* Add a bunch of public sponsors */
      int numSps = 10;
      int firstSp = addSponsors(numSps, true);

      /* Add a bunch of public events */
      int numEvs = 10;
      int firstEv = addEvents(numEvs, firstLoc, firstSp, true);
    } catch (Throwable t) {
      t.printStackTrace();
      fail("Exception getting service interface: " + t.getMessage());
    }

    assertTrue("dummy test 1", true);
  }

  /* ====================================================================
   *                       Private location methods.
   * ==================================================================== */

  /** Add a bunch of locations for later use
   *
   * @param n     int number of locations
   * @return int  key of first added (remainder should follow)
   */
  private int addLocations(int n,
                            boolean isPublic) {
    int first = -1;
    for (int i = 1; i <= n; i++) {
      int key = svciUtil.addLocation("Dummy location address " + i,
                                     "Dummy location subaddress " + i,
                                     "http://dummy.link" + i + ".edu",
                                     user,
                                     isPublic);

      if (key < 0) {
        // failure
        return key;
      }

      if (first < 0) {
        first = key;
      }
    }

    return first;
  }

  /* ====================================================================
   *                       Private sponsor methods.
   * ==================================================================== */

  /** Add a bunch of sponsors for later use
   *
   * @param n     int number of sponsors
   * @return int  key of first added (remainder should follow)
   */
  private int addSponsors(int n,
                          boolean isPublic) {
    int first = -1;
    for (int i = 1; i <= n; i++) {
      int key = svciUtil.addSponsor("Dummy sponsor name " + i,
                                    "Dummy sponsor phone " + i,
                                    "Dummy sponsor email " + i,
                                    "http://dummy.link" + i + ".edu",
                                    user,
                                    isPublic);

      if (key < 0) {
        // failure
        return key;
      }

      if (first < 0) {
        first = key;
      }
    }

    return first;
  }

  /* ====================================================================
   *                       Private keyword methods.
   * ==================================================================== */

  /** Add a bunch of sponsors for later use
   *
   * @param n     int number of sponsors
   * @return int  key of first added (remainder should follow)
   */
  private int addKeywords(int n,
                          boolean isPublic) {
    int first = -1;
    for (int i = 1; i <= n; i++) {
      int key = svciUtil.addKeyword("Dummy keyword " + i,
                                    "Dummy keyword description " + i,
                                    user,
                                    isPublic);

      if (key < 0) {
        // failure
        return key;
      }

      if (first < 0) {
        first = key;
      }
    }

    return first;
  }

  /* ====================================================================
   *                       Private event methods.
   * ==================================================================== */

  /** Add a bunch of events for later use
   *
   * @param n         int number of events
   * @param locn      int first location id
   * @param spn       int first sponsor id
   * @param isPublic  true to create public events
   * @return int      key of first added (remainder should follow)
   */
  private int addEvents(int n, int locn, int spn,
                        boolean isPublic) throws Throwable {
    int first = -1;
    for (int i = 1; i <= n; i++) {
      int key = svciUtil.addEvent("Dummy event " + i,
                                     "Dummy event description " + i,
                                     "2004-05-06 13:00:00",
                                     "2004-05-06 14:00:00",
                                     svciUtil.getLocation(locn - 1 + i, true),
                                     svciUtil.getSponsor(spn - 1 + i, true),
                                     null,
                                     "http://dummy.link" + i + ".edu",
                                     user,
                                     isPublic);

      if (key < 0) {
        // failure
        return key;
      }

      if (first < 0) {
        first = key;
      }
    }

    return first;
  }

  /* ====================================================================
   *                       Private methods.
   * ==================================================================== */

  private void log(String msg) {
    System.out.println(this.getClass().getName() + ": " + msg);
  }
}

