package edu.rpi.cct.uwcal.tools.dump;

import edu.washington.cac.calfacade.shared.AdminGroupVO;
import edu.washington.cac.calfacade.shared.AuthUserVO;
import edu.washington.cac.calfacade.shared.AuthUserPrefsVO;
import edu.washington.cac.calfacade.shared.EventVO;
import edu.washington.cac.calfacade.shared.EventRefVO;
import edu.washington.cac.calfacade.shared.filter.AliasFilterVO;
import edu.washington.cac.calfacade.shared.filter.AndFilterVO;
import edu.washington.cac.calfacade.shared.filter.CreatorFilterVO;
import edu.washington.cac.calfacade.shared.filter.FilterVO;
import edu.washington.cac.calfacade.shared.filter.KeywordFilterVO;
import edu.washington.cac.calfacade.shared.filter.LocationFilterVO;
import edu.washington.cac.calfacade.shared.filter.NotFilterVO;
import edu.washington.cac.calfacade.shared.filter.OrFilterVO;
import edu.washington.cac.calfacade.shared.filter.SponsorFilterVO;
import edu.washington.cac.calfacade.shared.KeywordAttrVO;
import edu.washington.cac.calfacade.shared.KeywordVO;
import edu.washington.cac.calfacade.shared.LocationVO;
import edu.washington.cac.calfacade.shared.SponsorVO;
import edu.washington.cac.calfacade.shared.UserVO;

import java.io.FileWriter;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.sql.Timestamp;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

/** Application to dump calendar data.
 *
 * @author Mike Douglass   douglm@rpi.edu
 * @version 1.0
 */
public class Dump implements DumpDefs {
  private boolean debug;

  DumpIntf di;

  /* Runtime arg -f Where we dump to.
   */
  private String fileName;

  /* runtime arg -i (id) */
  private String id = "sa";

  /* runtime arg -p (password) */
  private String pw = "";

  /* runtime arg -d  (driver) */
  private String driver = "org.hsqldb.jdbcDriver";

  /* runtime arg -u  (url) */
  private String url = "jdbc:hsqldb:hsql://localhost:8887";

  private int dbType = dbTypeHsql;

  String indent = "";

  private Writer out;

  /* Some counters */
  private int filters;
  private int users;

  private int subscribedUsers;

  private int subscriptions;

  private int locations;

  private int sponsors;

  private int keywords;
  private int keywordsWithAttrs;
  private int keywordAttrs;

  private int authusers;

  private int events;
  private int eventRefs;

  private int adminGroups;

  Dump() throws Throwable {
    di = new JdbcDump(url, driver, id, pw, dbType);
  }

  void open() throws Throwable {
    di.open();

    if (fileName == null) {
      out = new OutputStreamWriter(System.out);
    } else {
      out = new FileWriter(fileName);
    }
  }

  void close() throws Throwable {
    di.close();

    out.close();
  }

  void doDump() throws Throwable {
    tagStart(dumpTag);
    String saveIndent = indent;

    indent += "  ";

    dumpUsers();
    dumpLocations();
    dumpSponsors();
    dumpKeywords();
    dumpAuthUsers();
    dumpEvents();
    dumpEventRefs();
    dumpFilters();
    dumpAdminGroups();
    dumpDbLastmods();

    indent = saveIndent;
    tagEnd(dumpTag);
  }

  void stats() {
    System.out.println("            users: " + users);
    System.out.println("  subscribedUsers: " + subscribedUsers);
    System.out.println("    subscriptions: " + subscriptions);
    System.out.println("        locations: " + locations);
    System.out.println("         sponsors: " + sponsors);
    System.out.println("         keywords: " + keywords);
    System.out.println("keywordsWithAttrs: " + keywordsWithAttrs);
    System.out.println("     keywordAttrs: " + keywordAttrs);
    System.out.println("        authusers: " + authusers);
    System.out.println("           events: " + events);
    System.out.println("        eventRefs: " + eventRefs);
    System.out.println("          filters: " + filters);
    System.out.println("      adminGroups: " + adminGroups);
  }

  /** ===================================================================
   *                      Collection dump methods
   *  =================================================================== */

  void dumpUsers() throws Throwable {
    tagStart(sectionUsers);
    String saveIndent = indent;

    indent += "  ";

    Iterator it = di.getUsers();

    while (it.hasNext()) {
      UserVO u = (UserVO)it.next();

      dumpUser(u);
    }

    indent = saveIndent;
    tagEnd(sectionUsers);
  }

  void dumpLocations() throws Throwable {
    tagStart(sectionLocations);
    String saveIndent = indent;

    indent += "  ";

    Iterator it = di.getLocations();

    while (it.hasNext()) {
      LocationVO l = (LocationVO)it.next();

      dumpLocation(l);
    }

    indent = saveIndent;
    tagEnd(sectionLocations);
  }

  void dumpSponsors() throws Throwable {
    tagStart(sectionSponsors);
    String saveIndent = indent;

    indent += "  ";

    Iterator it = di.getSponsors();

    while (it.hasNext()) {
      SponsorVO s = (SponsorVO)it.next();

      dumpSponsor(s);
    }

    indent = saveIndent;
    tagEnd(sectionSponsors);
  }

  void dumpKeywords() throws Throwable {
    tagStart(sectionKeywords);
    String saveIndent = indent;

    indent += "  ";

    Iterator it = di.getKeywords();

    while (it.hasNext()) {
      KeywordVO k = (KeywordVO)it.next();

      dumpKeyword(k);
    }

    indent = saveIndent;
    tagEnd(sectionKeywords);
  }

  void dumpAuthUsers() throws Throwable {
    tagStart(sectionAuthUsers);
    String saveIndent = indent;

    indent += "  ";

    Iterator it = di.getAuthUsers();

    while (it.hasNext()) {
      AuthUserVO au = (AuthUserVO)it.next();

      dumpAuthUser(au);
    }

    indent = saveIndent;
    tagEnd(sectionAuthUsers);
  }

  void dumpEvents() throws Throwable {
    tagStart(sectionEvents);
    String saveIndent = indent;

    indent += "  ";

    Iterator it = di.getEvents();

    while (it.hasNext()) {
      EventVO e = (EventVO)it.next();

      dumpEvent(e);
    }

    indent = saveIndent;
    tagEnd(sectionEvents);
  }

  void dumpEventRefs() throws Throwable {
    tagStart(sectionEventRefs);
    String saveIndent = indent;

    indent += "  ";

    Iterator it = di.getEventRefs();

    while (it.hasNext()) {
      EventRefVO e = (EventRefVO)it.next();

      dumpEventRef(e);
    }

    indent = saveIndent;
    tagEnd(sectionEventRefs);
  }

  void dumpFilters() throws Throwable {
    tagStart(sectionFilters);
    String saveIndent = indent;

    indent += "  ";

    Iterator it = di.getFilters();

    while (it.hasNext()) {
      FilterVO f = (FilterVO)it.next();

      dumpFilter(f);
    }

    indent = saveIndent;
    tagEnd(sectionFilters);
  }

  void dumpAdminGroups() throws Throwable {
    tagStart(sectionAdminGroups);
    String saveIndent = indent;

    indent += "  ";

    Iterator it = di.getAdminGroups();

    while (it.hasNext()) {
      AdminGroupVO ag = (AdminGroupVO)it.next();

      dumpAdminGroup(ag);
    }

    indent = saveIndent;
    tagEnd(sectionAdminGroups);
  }

  void dumpDbLastmods() throws Throwable {
    tagStart(sectionDbLastmods);
    String saveIndent = indent;

    indent += "  ";

    Iterator it = di.getDbLastmods();

    while (it.hasNext()) {
      DumpDefs.DbLastmodVO lm = (DumpDefs.DbLastmodVO)it.next();

      dumpDbLastmod(lm);
    }

    indent = saveIndent;
    tagEnd(sectionDbLastmods);
  }

  /** ===================================================================
   *                      Single object dump methods
   *  =================================================================== */

  void dumpUser(UserVO u) throws Throwable {
    tagStart(objectUser);
    String saveIndent = indent;

    indent += "  ";

    taggedVal("id", u.getId());
    taggedVal("account", u.getAccount());
    taggedVal("created", u.getCreated());
    taggedVal("logon", u.getLogon());
    taggedVal("lastAccess", u.getLastAccess());
    taggedVal("lastModify", u.getLastModify());

    tagStart("subscriptions");
    String saveIndent2 = indent;

    indent += "  ";
    boolean subscribed = false;

    Collection s = u.getSubscriptions();
    Iterator si = s.iterator();

    while (si.hasNext()) {
      FilterVO f = (FilterVO)si.next();

      taggedVal("calendarid", f.getId());
      subscribed = true;
      subscriptions++;
    }

    if (subscribed) {
      subscribedUsers++;
    }

    indent = saveIndent2;
    tagEnd("subscriptions");

    indent = saveIndent;
    tagEnd(objectUser);

    users++;
  }

  void dumpLocation(LocationVO l) throws Throwable {
    tagStart(objectLocation);
    String saveIndent = indent;

    indent += "  ";

    taggedVal("id", l.getId());

    UserVO c = l.getCreator();

    if (c != null) {
      taggedVal("creator", c.getId());
    }
    taggedVal("public", l.getIsPublic());

    taggedVal("address", l.getAddress());
    taggedVal("subaddress", l.getSubaddress());
    taggedVal("link", l.getLink());

    indent = saveIndent;
    tagEnd(objectLocation);

    locations++;
  }

  void dumpSponsor(SponsorVO s) throws Throwable {
    tagStart(objectSponsor);
    String saveIndent = indent;

    indent += "  ";

    taggedVal("id", s.getId());

    UserVO c = s.getCreator();

    if (c != null) {
      taggedVal("creator", c.getId());
    }
    taggedVal("public", s.isPublic());

    taggedVal("name", s.getName());
    taggedVal("phone", s.getPhone());
    taggedVal("email", s.getEmail());
    taggedVal("link", s.getLink());

    indent = saveIndent;
    tagEnd(objectSponsor);

    sponsors++;
  }

  void dumpKeyword(KeywordVO k) throws Throwable {
    tagStart(objectKeyword);
    String saveIndent = indent;

    indent += "  ";

    taggedVal("id", k.getId());

    UserVO c = k.getCreator();

    if (c != null) {
      taggedVal("creator", c.getId());
    }
    taggedVal("public", k.isPublic());

    taggedVal("word", k.getWord());
    taggedVal("title", k.getTitle());
    taggedVal("description", k.getDescription());

    boolean hasAttrs = false;

    Set s = k.getAttributes();
    Iterator si = s.iterator();

    while (si.hasNext()) {
      tagStart("attribute");
      String saveIndent2 = indent;

      indent += "  ";

      KeywordAttrVO ka = (KeywordAttrVO)si.next();

      taggedVal("id", ka.getId());

      if (ka.getKeyword().getId() != k.getId()) {
        dmsg("Keyword id mismatch for keyword attrs for " + k);
      }
      taggedVal("name", ka.getName());
      taggedVal("val", ka.getVal());

      indent = saveIndent2;
      tagEnd("attribute");

      hasAttrs = true;
      keywordAttrs++;
    }

    if (hasAttrs) {
      keywordsWithAttrs++;
    }

    indent = saveIndent;
    tagEnd(objectKeyword);

    keywords++;
  }

  void dumpAuthUser(AuthUserVO au) throws Throwable {
    tagStart(objectAuthUser);
    String saveIndent = indent;

    indent += "  ";

    taggedVal("id", au.getUser().getId());
    taggedVal("userType", au.getUsertype());
    taggedVal("lastname", au.getLastname());
    taggedVal("firstname", au.getFirstname());
    taggedVal("phone", au.getPhone());
    taggedVal("email", au.getEmail());
    taggedVal("department", au.getDept());

    tagStart("preferences");
    String saveIndent2 = indent;

    indent += "  ";

    AuthUserPrefsVO aup = au.getPrefs();

    if (aup != null) {
      taggedVal("autoAddKeywords", aup.getAutoAddKeywords());
      taggedVal("autoAddLocations", aup.getAutoAddLocations());
      taggedVal("autoAddSponsors", aup.getAutoAddSponsors());

      Set s = aup.getPreferredKeywords();
      if ((s != null) && (s.size() > 0)) {
        tagStart("preferredKeywords");
        String saveIndent3 = indent;

        indent += "  ";
        Iterator si = s.iterator();

        while (si.hasNext()) {
          KeywordVO p = (KeywordVO)si.next();

          taggedVal("id", p.getId());
        }

        indent = saveIndent3;
        tagEnd("preferredKeywords");
      }

      s = aup.getPreferredLocations();
      if ((s != null) && (s.size() > 0)) {
        tagStart("preferredLocations");
        String saveIndent3 = indent;

        indent += "  ";
        Iterator si = s.iterator();

        while (si.hasNext()) {
          LocationVO p = (LocationVO)si.next();

          taggedVal("id", p.getId());
        }

        indent = saveIndent3;
        tagEnd("preferredLocations");
      }

      s = aup.getPreferredSponsors();
      if ((s != null) && (s.size() > 0)) {
        tagStart("preferredSponsors");
        String saveIndent3 = indent;

        indent += "  ";
        Iterator si = s.iterator();

        while (si.hasNext()) {
          SponsorVO p = (SponsorVO)si.next();

          taggedVal("id", p.getId());
        }

        indent = saveIndent3;
        tagEnd("preferredSponsors");
      }
    }

    indent = saveIndent2;
    tagEnd("preferences");

    indent = saveIndent;
    tagEnd(objectAuthUser);

    authusers++;
  }

  void dumpEvent(EventVO e) throws Throwable {
    tagStart(objectEvent);
    String saveIndent = indent;

    indent += "  ";

    taggedVal("id", e.getId());

    UserVO c = e.getCreator();

    if (c != null) {
      taggedVal("creator", c.getId());
    }
    taggedVal("public", e.getPublic());

    taggedVal("lastmod", e.getLastmod());
    taggedVal("seq", e.getSeq());
    taggedVal("created", e.getCreated());
    taggedVal("startdate", e.getStartdate());
    taggedVal("starttime", e.getStarttime());
    taggedVal("enddate", e.getEnddate());
    taggedVal("endtime", e.getEndtime());
    taggedVal("shortdesc", e.getShortdesc());
    taggedVal("longdesc", e.getLongdesc());
    taggedVal("link", e.getLink());

    LocationVO l = e.getLocation();
    if (l != null) {
      taggedVal("location", l.getId());
    }

    SponsorVO sp = e.getSponsor();
    if (sp != null) {
      taggedVal("sponsor", sp.getId());
    }

    // taggedVal("recurringStatus", e.getRecurringStatus());
    taggedVal("cost", e.getCost());

    tagStart("eventKeywords");

    String saveIndent2 = indent;

    indent += "  ";

    Vector keys = e.getKeywords();
    Iterator keysi = keys.iterator();

    while (keysi.hasNext()) {
      KeywordVO k = (KeywordVO)keysi.next();

      taggedVal("id", k.getId());
    }

    indent = saveIndent2;
    tagEnd("eventKeywords");

    indent = saveIndent;
    tagEnd(objectEvent);

    events++;
  }

  void dumpEventRef(EventRefVO e) throws Throwable {
    tagStart(objectEventRef);
    String saveIndent = indent;

    indent += "  ";

    taggedVal("eventid", e.getEventid());
    taggedVal("userid", e.getUser().getId());
    taggedVal("purpose", e.getPurpose());

    indent = saveIndent;
    tagEnd(objectEventRef);

    eventRefs++;
  }

  void dumpFilter(FilterVO f) throws Throwable {
    String tagName;

    int ref = -1;

    if (f instanceof AndFilterVO) {
      tagName = objectAndFilter;
    } else if (f instanceof KeywordFilterVO) {
      tagName = objectKeywordFilter;

      KeywordVO key = ((KeywordFilterVO)f).getKeyword();

      if (key != null) {
        ref = key.getId();
      }
    } else if (f instanceof CreatorFilterVO) {
      tagName = objectCreatorFilter;

      UserVO creator = ((CreatorFilterVO)f).getCreator();

      if (creator != null) {
        ref = creator.getId();
      }
    } else if (f instanceof LocationFilterVO) {
      tagName = objectLocationFilter;

      LocationVO l = ((LocationFilterVO)f).getLocation();

      if (l != null) {
        ref = l.getId();
      }
    } else if (f instanceof OrFilterVO) {
      tagName = objectOrFilter;
    } else if (f instanceof SponsorFilterVO) {
      tagName = objectSponsorFilter;

      SponsorVO s = ((SponsorFilterVO)f).getSponsor();

      if (s != null) {
        ref = s.getId();
      }
    } else if (f instanceof NotFilterVO) {
      tagName = objectNotFilter;

      FilterVO filter = ((NotFilterVO)f).getFilter();

      if (filter != null) {
        ref = filter.getId();
      }
    } else if (f instanceof AliasFilterVO) {
      tagName = objectAliasFilter;

      FilterVO filter = ((AliasFilterVO)f).getTarget();

      if (filter != null) {
        ref = filter.getId();
      }
    } else {
      throw new Exception("Unimplemented filter " + f.getClass().getName());
    }

    tagStart(tagName);
    String saveIndent = indent;

    indent += "  ";

    taggedVal("id", f.getId());
    taggedVal("name", f.getName());
    taggedVal("title", f.getTitle());
    taggedVal("description", f.getDescription());

    if (f.getParent() != null) {
      taggedVal("parent", f.getParent().getId());
    }

    if (ref > 0) {
      taggedVal("ref", ref);
    }

    indent = saveIndent;
    tagEnd(tagName);
    filters++;
  }

  void dumpAdminGroup(AdminGroupVO ag) throws Throwable {
    tagStart(objectAdminGroup);
    String saveIndent = indent;

    indent += "  ";

    taggedVal("groupname", ag.getName());
    taggedVal("description", ag.getDescription());

    UserVO u = ag.getGroupOwner();

    if (u != null) {
      taggedVal("groupOwner", u.getId());
    }

    u = ag.getOwner();

    if (u != null) {
      taggedVal("ownerid", u.getId());
    }

    tagStart("groupMembers");

    String saveIndent2 = indent;

    indent += "  ";

    Collection mbrs = ag.getGroupMembers();
    Iterator mbrsi = mbrs.iterator();

    while (mbrsi.hasNext()) {
      u = (UserVO)mbrsi.next();

      taggedVal("id", u.getId());
    }

    indent = saveIndent2;
    tagEnd("groupMembers");

    indent = saveIndent;
    tagEnd(objectAdminGroup);

    adminGroups++;
  }

  void dumpDbLastmod(DumpDefs.DbLastmodVO val) throws Throwable {
    tagStart(objectDbLastmod);
    String saveIndent = indent;

    indent += "  ";

    taggedVal("dbname", val.getDbname());
    taggedVal("lastmod", val.getLastmod());

    indent = saveIndent;
    tagEnd(objectDbLastmod);

    adminGroups++;
  }

  /** ===================================================================
   *                      Utility methods
   *  =================================================================== */

  void tagStart(String name) throws Throwable {
    out.write(indent + "<" + name + ">");
    out.write('\n');
  }

  void tagEnd(String name) throws Throwable {
    out.write(indent + "</" + name + ">");
    out.write('\n');
  }

  void taggedVal(String name, boolean val) throws Throwable {
    out.write(indent + "<" + name + ">" +
              val +
              "</" + name + ">");
    out.write('\n');
  }

  void taggedVal(String name, char val) throws Throwable {
    out.write(indent + "<" + name + ">" +
              val +
              "</" + name + ">");
    out.write('\n');
  }

  void taggedVal(String name, int val) throws Throwable {
    out.write(indent + "<" + name + ">" +
              val +
              "</" + name + ">");
    out.write('\n');
  }

  /** Anything for which toString() works OK
   */
  void taggedVal(String name, Object val) throws Throwable {
    if (val == null) {
      out.write(indent + "<" + name + ">" +
                "</" + name + ">");
    } else {
      String sval = String.valueOf(val);

      if ((sval.indexOf('&') < 0) && (sval.indexOf('<') < 0)) {
        out.write(indent + "<" + name + ">" +
                  sval +
                  "</" + name + ">");
      } else {
        out.write(indent + "<" + name + ">" +
                  "<![CDATA[" + sval + "]]>" +
                  "</" + name + ">");
      }
    }
    out.write('\n');
  }

  /*  void taggedVal(String name, Timestamp val) {
    System.out.println(indent + "<" + name + ">" +
                       val +
                       "</" + name + ">");
  }*/

  void processArgs(String[] args) throws Throwable {
    if (args == null) {
      return;
    }

    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-debug")) {
        debug = true;
      } else if (args[i].equals("-ndebug")) {
        debug = false;
      } else if (args[i].equals("-noarg")) {
      } else if (argpar("-d", args, i)) {
        i++;
        driver = args[i];
      } else if (argpar("-f", args, i)) {
        i++;
        fileName = args[i];
      } else if (argpar("-i", args, i)) {
        i++;
        id = args[i];
      } else if (argpar("-p", args, i)) {
        i++;
        pw = args[i];
      } else if (argpar("-u", args, i)) {
        i++;
        url = args[i];
      } else {
        dmsg("Illegal argument: " + args[i]);
        throw new Exception("Invalid args");
      }
    }
  }

  boolean argpar(String n, String[] args, int i) throws Exception {
    if (!args[i].equals(n)) {
      return false;
    }

    if ((i + 1) == args.length) {
      throw new Exception("Invalid args");
    }
    return true;
  }

  static void dmsg(String s) {
    System.out.println(s);
  }

  public static void main(String[] args) {
    Dump d = null;

    try {
      d = new Dump();

      d.processArgs(args);

      d.open();

      d.doDump();

      d.stats();

    } catch (Throwable t) {
      t.printStackTrace();
    } finally {
      try {
        d.close();
      } catch (Throwable t1) {
        t1.printStackTrace();
      }
    }
  }
}

