package edu.rpi.cct.uwcal.tools.dump;

import edu.washington.cac.calfacade.shared.UserVO;

import java.util.Iterator;

/** Interface which defines the dataabse functions needed to dump the
 * calendar database. It is intended that the implementation might cache
 * objects. Calling each method in the order below should be the most
 * efficient.
 *
 * <p>These methods return dummy objects for references. For example,
 * an event location is represented by a Locationbase object with only the
 * id filled in.
 *
 * <p>If the implementing class discovers a discrepency, e.g. a missing
 * user entry, it is up to the caller to determine that is the case.
 *
 * <p>Error messages should be emitted by the implementing classes.
 *
 * @author Mike Douglass   douglm@rpi.edu
 * @version 1.0
 */
public interface DumpIntf {
  /** Call after any init phase
   */
  public void open() throws Throwable;

  /** Call after dumping.
   */
  public void close();

  /** Will return an Iterator returning FilterBase objects.
   */
  public Iterator getFilters() throws Throwable;

  /** Will return an Iterator returning UserBase objects.
   * Subscriptions will be included
   */
  public Iterator getUsers() throws Throwable;

  /** Will return an Iterator returning LocationBase objects.
   */
  public Iterator getLocations() throws Throwable;

  /** Will return an Iterator returning SponsorBase objects.
   */
  public Iterator getSponsors() throws Throwable;

  /** Will return an Iterator returning KeywordBase objects.
   * Attributes will be attached
   */
  public Iterator getKeywords() throws Throwable;

  /** Will return an Iterator returning EventBase objects.
   * All relevent objects, keywords, locations, sponsors, creators will
   * be attached.
   */
  public Iterator getEvents() throws Throwable;

  /** Will return an Iterator returning EventRefBase objects.
   */
  public Iterator getEventRefs() throws Throwable;

  /** Will return an Iterator returning AuthUser objects. Preferences will
   * be attached - user objects will also be attached.
   */
  public Iterator getAuthUsers() throws Throwable;

  /** Will return an Iterator returning AdminGroupBase objects.
   */
  public Iterator getAdminGroups() throws Throwable;

  /** Will return an Iterator returning DbLastmodVO objects.
   */
  public Iterator getDbLastmods() throws Throwable;
}

