package edu.rpi.cct.uwcal.tools.dump;

import edu.washington.cac.calfacade.shared.AdminGroupVO;
import edu.washington.cac.calfacade.shared.AuthUserVO;
import edu.washington.cac.calfacade.shared.EventVO;
import edu.washington.cac.calfacade.shared.EventRefVO;
import edu.washington.cac.calfacade.shared.filter.AndFilterVO;
import edu.washington.cac.calfacade.shared.filter.CreatorFilterVO;
import edu.washington.cac.calfacade.shared.filter.FilterVO;
import edu.washington.cac.calfacade.shared.filter.KeywordFilterVO;
import edu.washington.cac.calfacade.shared.filter.LocationFilterVO;
import edu.washington.cac.calfacade.shared.filter.NotFilterVO;
import edu.washington.cac.calfacade.shared.filter.OrFilterVO;
import edu.washington.cac.calfacade.shared.filter.SponsorFilterVO;
import edu.washington.cac.calfacade.shared.KeywordAttrVO;
import edu.washington.cac.calfacade.shared.KeywordVO;
import edu.washington.cac.calfacade.shared.LocationVO;
import edu.washington.cac.calfacade.shared.SponsorVO;
import edu.washington.cac.calfacade.shared.UserVO;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

/** Application to restore from an XML calendar dump..
 *
 * @author Mike Douglass   douglm@rpi.edu
 * @version 1.0
 */
public class Restore extends ParseDump {
  private boolean concatdesc = false;

  private boolean printData = false;

  /** Runtime arg -sql filename If non-null we are generating sql
   */
  private SqlOut sqlOut;

  /** The dump file
   */
  private String fileName;

  /** If non-null we are updating the database via jdbc. Uses url, driver,
   * id and pw
   */
  private JdbcRestore jdbcRestore;

  /** Runtime arg -jdbc
   */
  private boolean jdbcUpdate;

  /* runtime arg -i (id) */
  private String id = "sa";

  /* runtime arg -p (password) */
  private String pw = "";

  /* runtime arg -u  (url) */
  private String url = "jdbc:hsqldb:hsql://localhost:8887";

  /* runtime arg -d  (driver) */
  private String driver = "org.hsqldb.jdbcDriver";

  private int dbType = dbTypeHsql;

  /* Some counters */
  private int filters;
  private int users;

  private int subscribedUsers;

  private int subscriptions;

  private int locations;

  private int sponsors;

  private int keywords;
  private int keywordsWithAttrs;
  private int keywordAttrs;

  private int authusers;

  private int events;
  private int eventRefs;

  private int adminGroups;

  private HashMap usersByIdTbl = new HashMap();
  private HashMap usersByAccountTbl = new HashMap();
  private HashMap locationsTbl = new HashMap();
  private HashMap sponsorsTbl = new HashMap();
  private HashMap keywordsTbl = new HashMap();
  private HashMap filtersTbl = new HashMap();

  private ArrayList subscriptionsList = new ArrayList();

  /** ===================================================================
   *                       Constructor
   *  =================================================================== */

  public Restore() {
  }

  /** ===================================================================
   *                       Restore methods
   *  =================================================================== */

  void open() throws Throwable {
    if (sqlOut != null) {
      sqlOut.open();
    }

    if (jdbcUpdate) {
      jdbcRestore = new JdbcRestore(url, driver, id, pw, dbType);
      jdbcRestore.open();
    }
  }

  void close() throws Throwable {
    if (sqlOut != null) {
      sqlOut.close();
    }

    if (jdbcRestore != null) {
      jdbcRestore.close();
    }
  }

  void doRestore() throws Throwable {
    parse(fileName);
  }

  void stats() {
    System.out.println("            users: " + users);
    System.out.println("  subscribedUsers: " + subscribedUsers);
    System.out.println("    subscriptions: " + subscriptions);
    System.out.println("        locations: " + locations);
    System.out.println("         sponsors: " + sponsors);
    System.out.println("         keywords: " + keywords);
    System.out.println("keywordsWithAttrs: " + keywordsWithAttrs);
    System.out.println("     keywordAttrs: " + keywordAttrs);
    System.out.println("        authusers: " + authusers);
    System.out.println("           events: " + events);
    System.out.println("        eventRefs: " + eventRefs);
    System.out.println("          filters: " + filters);
    System.out.println("      adminGroups: " + adminGroups);
  }

  /** ===================================================================
   *                       ParseDump methods
   *  =================================================================== */

  /** Called when we reach the end of the dump
   */
  public void endDump() {
    if (debug) {
      dmsg("endDump");
    }
  }

  /** Called when we start a new dump section. We can expect different
   * object classes
   */
  public void startSection(String name) {
    if (debug) {
      dmsg("startSection(" + name + ")");
    }
  }

  /** Called when we end a dump section.A start section should follow soon
   * after.
   */
  public void endSection(String name) throws Throwable {
    if (debug) {
      dmsg("endSection(" + name + ")");
    }
  }

  /** Called when an object has been restored. Any references to other
   * objects will be filled in with dummy objects defining the key. They
   * need to be replaced with the real objects either immediately if they
   * exist, or later in the restore process.
   */
  public void endObject(Object o) throws Throwable {
    if (printData) {
      dmsg("Restore object: " + o);
    }

    if (o instanceof AdminGroupVO) {
      restoreAdminGroup((AdminGroupVO)o);
    } else if (o instanceof AuthUserVO) {
      restoreAuthUser((AuthUserVO)o);
    } else if (o instanceof EventVO) {
      restoreEvent((EventVO)o);
    } else if (o instanceof EventRefVO) {
      restoreEventRef((EventRefVO)o);
    } else if (o instanceof FilterVO) {
      restoreFilter((FilterVO)o);
    } else if (o instanceof KeywordVO) {
      restoreKeyword((KeywordVO)o);
    } else if (o instanceof LocationVO) {
      restoreLocation((LocationVO)o);
    } else if (o instanceof SponsorVO) {
      restoreSponsor((SponsorVO)o);
    } else if (o instanceof UserVO) {
      restoreUser((UserVO)o);
    } else if (o instanceof DumpDefs.DbLastmodVO) {
      restoreDbLastmod((DumpDefs.DbLastmodVO)o);
    } else {
      dmsg("Unexpected object class: " + o);
    }
  }

  /** ===================================================================
   *                       private restore methods
   *  =================================================================== */

  /** ===================================================================
   *                       DumpDefs.DbLastmodVO
   *  =================================================================== */
  private void restoreDbLastmod(DumpDefs.DbLastmodVO o) throws Throwable {
    if (sqlOut != null) {
      sqlOut.restoreDbLastmod(o);
    }

    if (jdbcRestore != null) {
      jdbcRestore.restoreDbLastmod(o);
    }
  }

  /** ===================================================================
   *                       AdminGroupVO
   * Members are already added in the form of dummy entries.
   * Replace them with those we currently have in the user table..
   *  =================================================================== */
  private void restoreAdminGroup(AdminGroupVO o) throws Throwable {
    adminGroups++;

    o.setGroupOwner(getUser(o.getGroupOwner(), "group owner", o));
    o.setOwner(getUser(o.getOwner(), "owner", o));

    Collection mbrs = o.getGroupMembers();
    Vector nmbrs = new Vector();

    if (mbrs != null) {
      Iterator uit = mbrs.iterator();

      while (uit.hasNext()) {
        UserVO u = getUser((UserVO)uit.next(), "member", o);

        if (u != null) {
          nmbrs.add(u);
        }
      }
    }

    if (sqlOut != null) {
      sqlOut.restoreAdminGroup(o);
    }

    if (jdbcRestore != null) {
      jdbcRestore.restoreAdminGroup(o);
    }
  }

  /** ===================================================================
   *                       AuthUserVO
   *  =================================================================== */
  private void restoreAuthUser(AuthUserVO o) throws Throwable {
    authusers++;

    UserVO u = getUser(o.getUser(), "user", o);
    if (u != null) {
      o.setUser(u);
    }

    if (sqlOut != null) {
      sqlOut.restoreAuthUser(o);
    }

    if (jdbcRestore != null) {
      jdbcRestore.restoreAuthUser(o);
    }
  }

  /** ===================================================================
   *                       EventVO
   *  =================================================================== */
  private void restoreEvent(EventVO o) throws Throwable {
    o.setCreator(getCreator(o.getCreator(), o));
    events++;

    /* Replace the dummy location */
    LocationVO l = o.getLocation();

    if (l == null) {
      error("Missing dummy location for " + o);
    } else {
      l = (LocationVO)locationsTbl.get(new Integer(l.getId()));
      if (l == null) {
        error("Missing location for " + o);
      } else {
        o.setLocation(l);
      }
    }

    /* Replace the dummy sponsor */
    SponsorVO sp = o.getSponsor();

    if (sp == null) {
      error("Missing dummy sponsor for " + o);
    } else {
      sp = (SponsorVO)sponsorsTbl.get(new Integer(sp.getId()));
      if (sp == null) {
        error("Missing sponsor for " + o);
      } else {
        o.setSponsor(sp);
      }
    }

    /* Replace the dummy keywords */
    Vector kdummy = o.getKeywords();

    if ((kdummy != null) && (kdummy.size() > 0)) {
      Iterator it = kdummy.iterator();
      Vector keys = new Vector();

      while (it.hasNext()) {
        KeywordVO k = (KeywordVO)it.next();

        k = (KeywordVO)keywordsTbl.get(new Integer(k.getId()));
        if (k == null) {
          error("Missing keyword " + k.getId() + " for " + o);
        } else {
          keys.add(k);
        }
      }

      o.setKeywords(keys);
    }

    if (sqlOut != null) {
      sqlOut.restoreEvent(o);
    }

    if (jdbcRestore != null) {
      jdbcRestore.restoreEvent(o);
    }
  }

  /** ===================================================================
   *                       EventRefVO
   *  =================================================================== */
  private void restoreEventRef(EventRefVO o) throws Throwable {
    eventRefs++;

    if (sqlOut != null) {
      sqlOut.restoreEventRef(o);
    }

    if (jdbcRestore != null) {
      jdbcRestore.restoreEventRef(o);
    }
  }

  /** ===================================================================
   *                       FilterVO
   *  =================================================================== */
  private void restoreFilter(FilterVO o) throws Throwable {
    filtersTbl.put(new Integer(o.getId()), o);
    filters++;

    FilterVO pf = o.getParent();

    if (pf == null) {
      error("Missing dummy parent for " + o);
      pf = new FilterVO();
      o.setParent(pf);
    } else {
      int id = pf.getId();

      pf = (FilterVO)filtersTbl.get(new Integer(id));
      if (pf == null) {
        error("Missing parent " + id + " for " + o);
      } else {
        o.setParent(pf);
      }
    }

    if (o instanceof AndFilterVO) {
    } else if (o instanceof KeywordFilterVO) {
      KeywordVO k = ((KeywordFilterVO)o).getKeyword();

      if (k == null) {
        error("Missing dummy keyword for " + o);
      } else {
        int id = k.getId();

        k = (KeywordVO)keywordsTbl.get(new Integer(id));
        if (k == null) {
          error("Missing keyword " + id + " for " + o);
        } else {
          ((KeywordFilterVO)o).setKeyword(k);
        }
      }
    } else if (o instanceof CreatorFilterVO) {
      UserVO u = getUser(((CreatorFilterVO)o).getCreator(),
                           "creator", o);

      if (u != null) {
        ((CreatorFilterVO)o).setCreator(u);
      }
    } else if (o instanceof LocationFilterVO) {
      LocationVO l = ((LocationFilterVO)o).getLocation();

      if (l == null) {
        error("Missing dummy location for " + o);
      } else {
        int id = l.getId();

        l = (LocationVO)locationsTbl.get(new Integer(id));
        if (l == null) {
          error("Missing location " + id + " for " + o);
        } else {
          ((LocationFilterVO)o).setLocation(l);
        }
      }
    } else if (o instanceof OrFilterVO) {
    } else if (o instanceof SponsorFilterVO) {
      SponsorVO sp = ((SponsorFilterVO)o).getSponsor();

      if (sp == null) {
        error("Missing dummy sponsor for " + o);
      } else {
        int id = sp.getId();

        sp = (SponsorVO)sponsorsTbl.get(new Integer(id));
        if (sp == null) {
          error("Missing sponsor " + id + " for " + o);
        } else {
          ((SponsorFilterVO)o).setSponsor(sp);
        }
      }
    } else if (o instanceof NotFilterVO) {
      FilterVO f = ((NotFilterVO)o).getFilter();

      if (f == null) {
        error("Missing dummy filter for " + o);
      } else {
        int id = f.getId();

        f = (FilterVO)filtersTbl.get(new Integer(id));
        if (f == null) {
          error("Missing filter " + id + " for " + o);
        } else {
          ((NotFilterVO)o).setFilter(f);
        }
      }
    } else {
      throw new Exception("Unimplemented filter " + o);
    }

    if (sqlOut != null) {
      sqlOut.restoreFilter(o);
    }

    if (jdbcRestore != null) {
      jdbcRestore.restoreFilter(o);
    }
  }

  /** ===================================================================
   *                       KeywordVO
   *  =================================================================== */
  private void restoreKeyword(KeywordVO o) throws Throwable {
    keywordsTbl.put(new Integer(o.getId()), o);
    o.setCreator(getCreator(o.getCreator(), o));
    keywords++;

    Collection attrs = o.getAttributes();
    if ((attrs != null) && (attrs.size() > 0)) {
      keywordsWithAttrs++;
    }

    if (sqlOut != null) {
      sqlOut.restoreKeyword(o);
    }

    if (jdbcRestore != null) {
      jdbcRestore.restoreKeyword(o);
    }
  }

  /** ===================================================================
   *                       LocationVO
   *  =================================================================== */
  private void restoreLocation(LocationVO o) throws Throwable {
    locationsTbl.put(new Integer(o.getId()), o);
    o.setCreator(getCreator(o.getCreator(), o));
    locations++;

    if (sqlOut != null) {
      sqlOut.restoreLocation(o);
    }

    if (jdbcRestore != null) {
      jdbcRestore.restoreLocation(o);
    }
  }

  /** ===================================================================
   *                       SponsorVO
   *  =================================================================== */
  private void restoreSponsor(SponsorVO o) throws Throwable {
    sponsorsTbl.put(new Integer(o.getId()), o);
    o.setCreator(getCreator(o.getCreator(), o));
    sponsors++;

    if (sqlOut != null) {
      sqlOut.restoreSponsor(o);
    }

    if (jdbcRestore != null) {
      jdbcRestore.restoreSponsor(o);
    }
  }

  /** ===================================================================
   *                       UserVO
   *  =================================================================== */
  private void restoreUser(UserVO o) throws Throwable {
    usersByIdTbl.put(new Integer(o.getId()), o);
    usersByAccountTbl.put(o.getAccount(), o);

    users++;

    Collection s = o.getSubscriptions();

    if ((s != null) && (s.size() != 0)) {
      subscribedUsers++;
      subscriptions += s.size();

      subscriptionsList.add(o);
    }

    if (sqlOut != null) {
      sqlOut.restoreUser(o);
    }

    if (jdbcRestore != null) {
      jdbcRestore.restoreUser(o);
    }
  }

  private void restoreSubscriptions() throws Throwable {
    Iterator it = subscriptionsList.iterator();

    while (it.hasNext()) {
      UserVO o = (UserVO)it.next();

      Collection s = o.getSubscriptions();
      Vector newsubs = new Vector();

      Iterator subit = s.iterator();

      while (subit.hasNext()) {
        FilterVO f = (FilterVO)subit.next();

        int id = f.getId();

        f = (FilterVO)filtersTbl.get(new Integer(id));
        if (f == null) {
          error("Missing filter " + id + " for " + o);
          f = new FilterVO();
        } else {
          newsubs.add(f);
        }

        if (sqlOut != null) {
          sqlOut.subscribe(o, f);
        }

        if (jdbcRestore != null) {
          jdbcRestore.subscribe(o, f);
        }
      }

      o.setSubscriptions(newsubs);
    }
  }

  /** ===================================================================
   *                       private ParseDump methods
   *  =================================================================== */

  /** Replace the dummy object with a real one.
   */
  private UserVO getCreator(UserVO dummy, Object parent) {
    UserVO u = getUser(dummy, "creator", parent);

    if (u != null) {
      return u;
    }

    return dummy;
  }

  /** Replace the dummy parameter object with a real one.
   */
  private UserVO getUser(UserVO dummy, String fld, Object parent) {
    UserVO u;

    if (dummy == null) {
      error("Missing dummy " + fld + " for " + parent);
      return null;
    }

    u = (UserVO)usersByIdTbl.get(new Integer(dummy.getId()));
    if (u == null) {
      error("Missing " + fld + " for " + parent);
      return null;
    }

    return u;
  }

  boolean processArgs(String[] args) throws Throwable {
    if (args == null) {
      return true;
    }

    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-debug")) {
        debug = true;
      } else if (args[i].equals("-ndebug")) {
        debug = false;
      } else if (args[i].equals("-noarg")) {
        debug = false;
      } else if (args[i].equals("-concatdesc")) {
        concatdesc = true;
      } else if (args[i].equals("-printdata")) {
        printData = true;
      } else if (args[i].equals("-jdbc")) {
        jdbcUpdate = true;
      } else if (args[i].equals("-njdbc")) {
        jdbcUpdate = false;
      } else if (argpar("-d", args, i)) {
        i++;
        driver = args[i];
      } else if (argpar("-i", args, i)) {
        i++;
        id = args[i];
      } else if (argpar("-p", args, i)) {
        i++;
        pw = args[i];
      } else if (argpar("-u", args, i)) {
        i++;
        url = args[i];
      } else if (argpar("-sql", args, i)) {
        i++;
        sqlOut = new SqlOut(args[i]);
      } else if (args[i].equals("-nsql")) {
        sqlOut = null;
      } else if (argpar("-f", args, i)) {
        i++;
        fileName = args[i];
      } else {
        dmsg("Illegal argument: " + args[i]);
        usage();
        return false;
      }
    }

    return true;
  }

  void usage() {
    System.out.println("Usage:");
    System.out.println("arsg   -debug");
    System.out.println("       -ndebug");
    System.out.println("       -f filename");
    System.out.println("            define name of input file");
    System.out.println("       -concatdesc");
    System.out.println("            put all event description in one field");
    System.out.println("       -jdbc");
    System.out.println("            Update the database via jdbc");
    System.out.println("       -printdata");
    System.out.println("            print the data before restore");
    System.out.println("       -sql filename");
    System.out.println("            generate sql into named file");
    System.out.println("");
  }

  boolean argpar(String n, String[] args, int i) throws Exception {
    if (!args[i].equals(n)) {
      return false;
    }

    if ((i + 1) == args.length) {
      throw new Exception("Invalid args");
    }
    return true;
  }

  public static void main(String[] args) {
    Restore r = null;

    try {
      r = new Restore();

      if (!r.processArgs(args)) {
        return;
      }

      r.open();

      r.doRestore();

      r.close();

      r.stats();

    } catch (Throwable t) {
      t.printStackTrace();
    } finally {
      try {
        r.close();
      } catch (Throwable t1) {
        t1.printStackTrace();
      }
    }
  }
}

